/**
 * srvoa - soa infrastructure for node js
 *
 * @copyright   Copyright (c) 2015, Alrik Zachert
 * @license     https://gitlab.com/a-z/node-srvoa/blob/master/LICENSE BSD-2-Clause
 */

"use strict";

/**
 * The srvoa service manager
 */

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ServiceManager = (function () {
  /**
   * The map of key => service.
   *
   * @property services {Object}
   */

  /**
   * Initialize services map.
   *
   * @constructor
   */

  function ServiceManager() {
    _classCallCheck(this, ServiceManager);

    this.services = {};
  }

  /**
   * Lookup and return a service by its key.
   *
   * @param   {String} key
   * @return  {Object}|{undefined}
   */

  _createClass(ServiceManager, [{
    key: "get",
    value: function get(key) {
      return this.services[key];
    }

    /**
     * Store a service for the given key.
     *
     * @param   {String} key
     * @param   {Object} service
     * @return  {ServiceManager}
     */
  }, {
    key: "set",
    value: function set(key, service) {
      this.services[key] = service;

      return this;
    }

    /**
     * Delete the service for the given key.
     *
     * @param   {String} key
     * @return  {ServiceManager}
     */
  }, {
    key: "remove",
    value: function remove(key) {
      delete this.services[key];

      return this;
    }

    /**
     * Check for the existence of a service by its key.
     *
     * @param   {String} key
     * @return  {Boolean}
     */
  }, {
    key: "has",
    value: function has(key) {
      return key in this.services;
    }
  }]);

  return ServiceManager;
})();

module.exports = ServiceManager;